/* createTestFile.c
 *
 * This file is part of the "anon-tar" package.
 *
 * The "anon-tar" package may be used under the terms and conditions of the
 * GNU General Public License version 2. A copy of these is included in the
 * file "COPYING" in the "doc" subdirectory.
 *
 * (C) 2003 Peter Conrad <conrad@unix-ag.uni-kl.de>
 */
 
/* This program creates a sparse file (given as the first argument) of random
 * length and content.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#define VERSION "createTestFile V0.1 (C) 2003 Peter Conrad <conrad@unix-ag.uni-kl.de"

static void usage(char *progname, int exitcode) {
char *USAGE = "Usage: ";
char *OPTIONS = " [-h | --help] [--version]\n";
 
    write(STDERR_FILENO, USAGE, strlen(USAGE));
    write(STDERR_FILENO, progname, strlen(progname));
    write(STDERR_FILENO, OPTIONS, strlen(OPTIONS));
 
    exit(exitcode);
}

int main(int argc, char **argv) {
char *UNKN_OPTION = "Unknown option: ";
char *UNKN_ARG = "Unknown argument: ";
char *CANT_OPEN = "Can't open output file: ";
char *SEEK_FAIL = "Seek failed: ";
char *TRUNC_FAIL = "Truncate failed: ";
char *INCOMPLETE_WRITE = "Incomplete write! Aborting...\n";
int i, out, size = 0;
 
    for (i = 1; i < argc && argv[i][0] == '-'; i++) {
        if (!strcmp(argv[i], "-h") || !strcmp(argv[i], "--help")) {
            usage(argv[0], 0);
        } else if (!strcmp(argv[i], "--version")) {
            write(STDOUT_FILENO, VERSION, strlen(VERSION));
            exit(0);
        } else {
            write(STDERR_FILENO, UNKN_OPTION, strlen(UNKN_OPTION));
            write(STDERR_FILENO, argv[i], strlen(argv[i]));
            write(STDERR_FILENO, "\n", 1);
            usage(argv[0], 1);
        }
    }

    if (i != argc -1) {
	write(STDERR_FILENO, UNKN_ARG, strlen(UNKN_ARG));
        write(STDERR_FILENO, argv[i], strlen(argv[i]));
        write(STDERR_FILENO, "\n", 1);
        usage(argv[0], 1);
    }

    if ((out = open(argv[i], O_WRONLY | O_CREAT | O_TRUNC, 0666)) < 0) {
	char *err = strerror(errno);
	write(STDERR_FILENO, CANT_OPEN, strlen(CANT_OPEN));
	write(STDERR_FILENO, err, strlen(err));
	write(STDERR_FILENO, "\n", 1);
	exit(1);
    }

    srand(time(NULL) ^ getpid());
    for (i = 0; i < 10; i++) {
        int bytes = rand() & 16383, j;
	char dummy;

	if (lseek(out, 0, SEEK_END) != size) {
	    char *err = strerror(errno);
	    write(STDERR_FILENO, SEEK_FAIL, strlen(SEEK_FAIL));
	    write(STDERR_FILENO, err, strlen(err));
	    write(STDERR_FILENO, "\n", 1);
	    exit(1);
	}
	
        for (j = 0; j < bytes; j++) {
	    dummy = rand() & 255;
	    if (write(out, &dummy, 1) != 1) {
	        char *err = strerror(errno);
	        write(STDERR_FILENO, INCOMPLETE_WRITE,strlen(INCOMPLETE_WRITE));
	        write(STDERR_FILENO, err, strlen(err));
	        write(STDERR_FILENO, "\n", 1);
	        exit(1);
	    }
        }
        size += bytes;
        bytes = rand() & 16383;
        size += bytes;
        if (ftruncate(out, size)) {
	    char *err = strerror(errno);
	    write(STDERR_FILENO, TRUNC_FAIL, strlen(TRUNC_FAIL));
	    write(STDERR_FILENO, err, strlen(err));
	    write(STDERR_FILENO, "\n", 1);
	    exit(1);
	}
    }
    close(out);
}
