#
# new-freenet-project - set up a directory structure for a new in-freenet project
################################################################
# Copyright (C) 2003  Peter Conrad <conrad@unix-ag.uni-kl.de>
#
# See the file "COPYING" for further information about
# the copyright and warranty status of this work.

if test $# -ne 0 ; then
 
  for opt in "$@" ; do
    case $opt in
 
      --version|-V) exec larch --version
            ;;
 
      --help|-h)
            printf "Set up a directory structure for a new in-freenet project\\n"
            printf "Usage: new-freenet-project [options] [ssk-public-key] project-name\\n"
            printf "\\n"
            printf " -V --version           print version info\\n"
            printf " -h --help              display help\\n"
            printf "\\n"
            printf " -g                     generate an SSK key pair\\n"
            printf "\\n"
            printf "Specifying an SSK public key and option -g are mutually exclusive.\\n"
            printf "However, one of both is required.\\n"
            printf "\\n"
            printf "This will create (in the current directory) the following structure:\\n"
            printf "\\n"
            printf "./Projects/<project-name>/bin/\\n"
            printf "                         /src/\\n"
            printf "./Archives/{Revisions}/\\n"
            printf "./p<n>.sh\\n"
            printf "\\n"
            printf "p<n>.sh is a shell script that sets your $HOME to /Projects/<project-name>/ .\\n"
            printf "Execute it before you work on the project. <n> will be a number that is\\n"
            printf "incremented with every project you add in the current dir.\\n"
            printf "\\n"
            printf "In addition, new-freenet-project will create an arch ID and a default\\n"
            printf "archive in ./Archives/<project-name>/ for you.\\n"
            printf "\\n"
            printf "If you specify the -g option, a new SSK key pair will be generated using\\n"
            printf "fcptools. The keys will be in the new $HOME/.project-keys .\\n"
            printf "\\n"
            printf "With this command, everything should be set up for your new anonymous software\\n"
            printf "project!\\n"
            exit 0
            ;;

      *)
            ;;
    esac
  done
fi

if [ "$#" != 2 ]; then
    echo "Usage: new-freenet-project [options] [ssk-public-key] project-name"
    echo "Use option -h or --help for a detailed explanation."
    exit 1
fi

umask 077;

mkdir -p "Projects/$2/bin"
mkdir -p "Projects/$2/src"
mkdir -p "Archives/{Revisions}"

if [ "$1" = "-g" ]; then
    echo Generating key pair...
    ssktool --sskfile "Projects/$2/.project-keys" --genkeys
    while [ `grep -c 'Public key:  .*~' "Projects/$2/.project-keys"` -gt 0 ]; do
	echo Retrying...
	rm -f "Projects/$2/.project-keys"
	ssktool --sskfile "Projects/$2/.project-keys" --genkeys
    done
    echo Keys saved to "Projects/$2/.project-keys"
    ssk=`grep 'Public key:' "Projects/$2/.project-keys" | cut -c 14-`
else
    ssk=`echo "$1" | sed 's=^freenet:==;s=SSK@==;s=PAgM/.*=='`
    echo "Public-key:  $ssk" >"Projects/$2/.project-keys"
fi

( HOME=`pwd`/"Projects/$2"
  export HOME 
  larch my-id "<$2@$ssk.freenet>"
  larch make-archive "$2@$ssk.freenet--$2" "Archives/$2"
  larch my-default-archive "$2@$ssk.freenet--$2"
  larch my-revision-library `pwd`/"Archives/{Revisions}"
  larch register-archive "$2@$ssk.freenet--$2-mirror" "freenet:SSK@${ssk}PAgM/$2"
)

i=1
while [ -f p$i.sh ]; do
    i=$(($i + 1))
done

cat >p$i.sh <<__EOT__
#!/bin/sh

oldhome=\$HOME
HOME=`pwd`/"Projects/$2"
PATH=\$HOME/bin:\$PATH
export HOME PATH
cd \$HOME
\$SHELL -i
echo HOME is now \$oldhome
__EOT__

chmod 755 p$i.sh

echo Execute `pwd`/p$i.sh to enter the project environment.

cat >"Archives/$2/index.html" <<__EOT__
<html>
<head><title>$2</title></head>
<body>
<h1>$2</h1>

<p>
Welcome to this cool new project! Use the following archive name to retrieve
the sources:
</p><p>
<code>$2@$ssk.freenet--$2</code>
</p>

</body>
</html>
__EOT__

exit 0

# tag: fa00f8db04d189b590787ee866c86fca new-freenet-project.sh
#
