/* cfgcat.c
 *
 ****************************************************************
 * Copyright (C) 2003 Tom Lord
 * 
 * See the file "COPYING" for further information about
 * the copyright and warranty status of this work.
 */


#include "config-options.h"
#include "hackerlab/cmd/main.h"
#include "file-utils/libarch/namespace.h"
#include "file-utils/libarch/project-tree.h"
#include "file-utils/libarch/configs.h"



static t_uchar * program_name = "cfgcat";
static t_uchar * usage = "[options] config";
static t_uchar * version_string = (cfg__std__package " from regexps.com\n"
				   "\n"
				   "Copyright 2003 Tom Lord\n"
				   "\n"
				   "This is free software; see the source for copying conditions.\n"
				   "There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A\n"
				   "PARTICULAR PURPOSE.\n"
				   "\n"
				   "Report bugs to " cfg__file_utils_bug_mail ".\n"
				   "\n");

#define OPTS(OP, OP2) \
  OP (opt_help_msg, "h", "help", 0, \
      "Display a help message and exit.") \
  OP (opt_long_help, "H", 0, 0, \
      "Display a verbose help message and exit.") \
  OP (opt_version, "V", "version", 0, \
      "Display a release identifier string") \
  OP2 (opt_version, 0, 0, 0, "and exit.") \
  OP (opt_dir, "d", "dir DIR", 1, \
      "cd to DIR first") \
  OP (opt_output, "o", "output CFG", 1, \
      "write the output as config CFG") \
  OP (opt_force, "f", "force", 0, \
      "overwrite an exiting config (with --output)") \
  OP (opt_snap, "s", "snap", 0, \
      "Show current patch levels of subtree packages.")

static t_uchar long_help[] = ("output information about a multi-project config\n");

enum options
{
  OPTS (OPT_ENUM, OPT_IGN)  
};

struct opt_desc opts[] = 
{
  OPTS (OPT_DESC, OPT_DESC)
    {-1, 0, 0, 0, 0}
};



int
main (int argc, char * argv[])
{
  int o;
  struct opt_parsed * option;
  t_uchar * dir = 0;
  int snap = 0;
  int force = 0;
  t_uchar * output = 0;

  dir = str_save (0, ".");

  safe_buffer_fd (1, 0, O_WRONLY, 0);

  option = 0;

  while (1)
    {
      o = opt_standard (lim_use_must_malloc, &option, opts, &argc, argv, program_name, usage, version_string, long_help, opt_help_msg, opt_long_help, opt_version);
      if (o == opt_none)
	break;
      switch (o)
	{
	default:
	  safe_printfmt (2, "unhandled option `%s'\n", option->opt_string);
	  panic ("internal error parsing arguments");

	usage_error:
	  opt_usage (2, argv[0], program_name, usage, 1);
	  exit (1);

          /* bogus_arg: */
	  safe_printfmt (2, "ill-formed argument for `%s' (`%s')\n", option->opt_string, option->arg_string);
	  goto usage_error;

        case opt_dir:
          {
            lim_free (0, dir);
            dir = str_save (0, option->arg_string);
            break;
          }

        case opt_snap:
          {
            snap = 1;
            break;
          }

        case opt_force:
          {
            force = 1;
            break;
          }

        case opt_output:
          {
            output = str_save (0, option->arg_string);
            break;
          }
	}
    }

  if (argc != 2)
    goto usage_error;

  {
    t_uchar * config;
    t_uchar * tree_root = 0;
    rel_table config_list = 0;

    config = argv[1];

    if (!arch_valid_config_name (config))
      {
        safe_printfmt (2, "cfgcat: invalid config name (%s)\n", config);
        exit (2);
      }

    if (output && !arch_valid_config_name (config))
      {
        safe_printfmt (2, "cfgcat: invalid config name (%s)\n", output);
        exit (2);
      }

    tree_root = arch_tree_root (0, dir, 0);

    if (!tree_root)
      {
        safe_printfmt (2, "cfgcat: not in project tree (%s)\n", dir);
        exit (1);
      }

    config_list = arch_read_config (tree_root, config);


    {
      int out_fd;

      if (output)
        out_fd = arch_begin_new_config (tree_root, output, force);
      else
        out_fd = 1;

      if (!snap)
        rel_print_table (out_fd, config_list);
      else
        {
          rel_table snap = 0;
          
          snap = arch_config_from_tree (tree_root, config_list);
          rel_print_table (out_fd, snap);

          rel_free_table (snap);
        }

      if (output)
        {
          arch_finish_new_config (out_fd, tree_root, output, force);
          safe_printfmt (1, "* wrote config %s\n", output);
        }

      rel_free_table (config_list);
      lim_free (0, tree_root);
    }
  }

  lim_free (0, dir);
  lim_free (0, output);
  return 0;
}



/* tag: Tom Lord Fri May 30 02:12:37 2003 (cfgcat.c)
 */
