/* register-archive.c
 *
 ****************************************************************
 * Copyright (C) 2003 Tom Lord
 * 
 * See the file "COPYING" for further information about
 * the copyright and warranty status of this work.
 */


#include "config-options.h"
#include "hackerlab/cmd/main.h"
#include "file-utils/libfsutils/ensure-dir.h"
#include "file-utils/libarch/my.h"
#include "file-utils/libarch/namespace.h"
#include "file-utils/libarch/archives.h"



static t_uchar * program_name = "register-archive";
static t_uchar * usage = "[options] archive location";
static t_uchar * version_string = (cfg__std__package " from regexps.com\n"
				   "\n"
				   "Copyright 2003 Tom Lord\n"
				   "\n"
				   "This is free software; see the source for copying conditions.\n"
				   "There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A\n"
				   "PARTICULAR PURPOSE.\n"
				   "\n"
				   "Report bugs to " cfg__file_utils_bug_mail ".\n"
				   "\n");

#define OPTS(OP, OP2) \
  OP (opt_help_msg, "h", "help", 0, \
      "Display a help message and exit.") \
  OP (opt_long_help, "H", 0, 0, \
      "Display a verbose help message and exit.") \
  OP (opt_version, "V", "version", 0, \
      "Display a release identifier string") \
  OP2 (opt_version, 0, 0, 0, "and exit.") \
  OP (opt_force, "f", "force", 0, \
      "overwrite existing location") \
  OP (opt_delete, "d", "delete", 0, \
      "delete archive registration")

static t_uchar long_help[] = ("change an archive location registration\n");

enum options
{
  OPTS (OPT_ENUM, OPT_IGN)  
};

struct opt_desc opts[] = 
{
  OPTS (OPT_DESC, OPT_DESC)
    {-1, 0, 0, 0, 0}
};



int
main (int argc, char * argv[])
{
  int o;
  struct opt_parsed * option;
  int delete;
  int force;

  delete = 0;
  force = 0;

  safe_buffer_fd (1, 0, O_WRONLY, 0);

  option = 0;

  while (1)
    {
      o = opt_standard (lim_use_must_malloc, &option, opts, &argc, argv, program_name, usage, version_string, long_help, opt_help_msg, opt_long_help, opt_version);
      if (o == opt_none)
	break;
      switch (o)
	{
	default:
	  safe_printfmt (2, "unhandled option `%s'\n", option->opt_string);
	  panic ("internal error parsing arguments");

	usage_error:
	  opt_usage (2, argv[0], program_name, usage, 1);
	  exit (1);

          /* bogus_arg: */
	  safe_printfmt (2, "ill-formed argument for `%s' (`%s')\n", option->opt_string, option->arg_string);
	  goto usage_error;

        case opt_delete:
          {
            delete = 1;
            break;
          }

        case opt_force:
          {
            force = 1;
            break;
          }
	}
    }

  if (delete ? (argc != 2) : (argc != 3))
    goto usage_error;

  if (delete)
    {
      t_uchar * archive_name;

      archive_name = argv[1];

      if (!arch_valid_archive_name (archive_name))
        {
          safe_printfmt (2, "register-archive: invalid archive name (%s)\n", archive_name);
          exit (1);
        }

      arch_delete_archive_location (archive_name);
    }
  else
    {
      t_uchar * archive_name;
      t_uchar * archive_location;

      archive_name = argv[1];
      archive_location = argv[2];

      if (!arch_valid_archive_name (archive_name))
        {
          safe_printfmt (2, "register-archive: invalid archive name (%s)\n", archive_name);
          exit (1);
        }

      arch_set_archive_location (archive_name, archive_location, force);
    }

  return 0;
}




/* tag: Tom Lord Sun May 18 20:34:28 2003 (register-archive.c)
 */
